/*
 * Created on Sep 18, 2004
 */
package org.tekkotsu.ui.editor.model;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.geometry.Point;

public abstract class AbstractConnectionModel extends AbstractModel {
	private StateNodeModel source, target;

	private List<Point> bendpoints   = new ArrayList<Point>();

	public static final String P_BEND_POINT = "_bend_point";

	public AbstractConnectionModel(List<Point> bendpoints) {
		if( bendpoints != null ) {
			for(int i = 0; i < bendpoints.size(); ++i) {
				Point point = bendpoints.get(i);
				if( point.y < 4 ) point.y = 4;
				if( point.x < 4 ) point.x = 4;
				this.bendpoints.add(bendpoints.get(i));
			}
		}
	}
	
	public List<Point> getBendpoints() {
		return bendpoints;
	}
	
	public void addBendpoint(int index, Point point) {
		if( point.y < 4 ) point.y = 4;
		if( point.x < 4 ) point.x = 4;
 		bendpoints.add(index, point);
		props.firePropertyChange(P_BEND_POINT, null, null);
	}

	public void removeBendpoint(int index) {
		bendpoints.remove(index);
		props.firePropertyChange(P_BEND_POINT, null, null);
	}

	public void replaceBendpoint(int index, Point point) {
		if( point.y < 4 ) point.y = 4;
		if( point.x < 4 ) point.x = 4;
		bendpoints.set(index, point);
		props.firePropertyChange(P_BEND_POINT, null, null);
	}

	@Override
	public boolean isPreview() {
		AbstractModel parent = getParent();
		
		return parent == null ? false : parent.isPreview();
	}

	public void attachSource() {
		if (!source.getModelSourceConnections().contains(this))
			source.addSourceConnection(this);
	}

	public void attachTarget() {
		if (!target.getModelTargetConnections().contains(this))
			target.addTargetConnection(this);
	}

	public void detachSource() {
		source.removeSourceConnection(this);
	}

	public void detachTarget() {
		target.removeTargetConnection(this);
	}

	public StateNodeModel getSource() {
		return source;
	}

	public StateNodeModel getTarget() {
		return target;
	}

	public void setSource(StateNodeModel model) {
		source = model;
	}

	public void setTarget(StateNodeModel model) {
		target = model;
	}
	
	public String partialToString( StateNodeModel snm ) {
		return toString();
	}

	public void clearBendpoints() {
		bendpoints.clear();
		props.firePropertyChange(P_BEND_POINT, null, null);
	}
}
