package org.tekkotsu.ui.editor.model;

/**
 * @author asangpet
 */

import static org.tekkotsu.ui.editor.resources.IDTag.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.jface.resource.ColorRegistry;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.ui.views.properties.ColorPropertyDescriptor;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.jdom.Attribute;
import org.jdom.Content;
import org.jdom.Element;
import org.tekkotsu.ui.editor.model.commands.DeleteConnectionCommand;
import org.tekkotsu.ui.editor.resources.IDTag;
import org.tekkotsu.ui.util.Debugger;

public class StateNodeModel extends AbstractModel
{	
    public static final String P_CLASS = "_class";

    public static final String P_COLOR = "_color";

    public static final String P_BLEND_COLOR = "_blend_color";

    public static final String P_CONSTRAINT = "_constraint";

    public static final String P_ID = "_id";

    public static final String P_INTRANS = "_intrans";

    public static final String P_OUTTRANS = "_outtrans";

    public static final String P_LABEL = "_label";

    public static final String P_SHAPE = "_shape";

    public static final String P_SHAPE_STYLE_ELLIPSE = "Ellipse";

    public static final String P_SHAPE_STYLE_RECTANGLE = "Rectangle";

    public static final String P_SHAPE_STYLE_ROUNDED_RECTANGLE = "Rounded Rectangle";

    public static final String P_SHAPE_STYLE_HEXAGON = "Hexagon";

    public static final String P_SOURCE_CONNECTION = "_source_connection";

    public static final String P_TARGET_CONNECTION = "_target_connection";

    public static final String P_ACTIVATE = "_state_activate";

    public static final String P_DEACTIVATE = "_state_deactivate";

    private static final String[] shapeValues = { P_SHAPE_STYLE_RECTANGLE,
            P_SHAPE_STYLE_ELLIPSE, P_SHAPE_STYLE_ROUNDED_RECTANGLE,
            P_SHAPE_STYLE_HEXAGON };

    protected static int nextStateID = 1;
    
    protected int stateID; 

    private ColorRegistry colorRegistry = new ColorRegistry();

    private Rectangle constraint;

    private String id = "State";

    private String label = "State";

    private String shape = P_SHAPE_STYLE_RECTANGLE;

    private int shapeIndex = 0;

    private SourceObjectModel source;
    
    protected LayoutData parent;

    protected List<AbstractConnectionModel> sourceConnections = new ArrayList<AbstractConnectionModel>();

    protected List<AbstractConnectionModel> targetConnections = new ArrayList<AbstractConnectionModel>();

    public StateNodeModel()
    {
        super();
        setId(nextStateID++);
        setLabel(getId());
    }

    public StateNodeModel(Element xml, LayoutData parent)
    {
        this();
        setParent(parent);
        assert (XML_state_tag.equals(xml.getName()));
        parseXML(xml);
    }
    
    public void parseXML(Element xml)
    {
    	String idTag = xml.getAttributeValue(IDTag.XML_common_id);
        if (!((idTag == null) || (idTag.equals("")))) setId(idTag);
        
        if( xml.getName().equals(IDTag.XML_state_tag))
        {
        	setSource(parent.getModelData().getNode(idTag));
        }
        else if( xml.getName().equals(IDTag.XML_transition_tag) )
        {
        	setSource(parent.getModelData().getTransition(idTag));
        }
        else
        {
        	throw new IllegalArgumentException();
        }
    
        if (xml.getAttributeValue(IDTag.XML_common_label) == null)
            setLabel(getId());
        else
            setLabel(xml.getAttributeValue(IDTag.XML_common_label));

        RGB color = IDTag
                .tag2RGB(xml.getAttributeValue(IDTag.XML_common_color));
        setColor(color);
        try
        {        	
        	Attribute aTop = xml.getAttribute(IDTag.XML_common_top);
        	Attribute aLeft = xml.getAttribute(IDTag.XML_common_left);
        	Attribute aWidth = xml.getAttribute(IDTag.XML_common_width);
        	Attribute aHeight = xml.getAttribute(IDTag.XML_common_height);
        	
        	if( aTop != null && aLeft != null && aWidth != null && aHeight != null)
        	{
	            int y = aTop.getIntValue();
	            int x = aLeft.getIntValue();
	            int width = aWidth.getIntValue();
	            int height = aHeight.getIntValue();
	
	            constraint = new Rectangle(x, y, width, height);
	            setConstraint(constraint);
        	}
        }
        catch (Exception e)
        {
        	Debugger.printThrowable(e);
        }
        try
        {
        	Attribute aShape = xml.getAttribute(IDTag.XML_state_shape);
        	
        	if( aShape != null )
        		setShape(aShape.getValue());
        }
        catch (Exception e)
        {
        	Debugger.printThrowable(e);
        }
    }

    public StateNodeModel(SourceNodeModel source)
    {
        this();
        setId(source.getId());
        setLabel(source.getId());
        setSource(source);

        ModelData parent = source.getParentModel();
        int SIZE = parent.getNodes().size();
        
        float hue = .97f * (stateID % SIZE) / SIZE;
		float brightness = 0.6f + 0.2f * (stateID % 2);
		float saturation = 0.8f;
		
		java.awt.Color color = java.awt.Color.getHSBColor(hue,saturation,brightness);
        
        RGB rgb = new RGB(color.getRed(), color.getGreen(), color.getBlue());
        colorRegistry.put("Color" + stateID, rgb);
        setColor(colorRegistry.getRGB("Color" + stateID));
    }

    public void verifySource(ModelData src)
    {
        Debugger.printDebug(Debugger.DEBUG_ALL, "Verify src:" + id + ":"
                + src.getNode(id));
        if (src.getChild(id) == null)
        {
        	props.firePropertyChange(LayoutData.P_SOURCE_EXIST, null,Boolean.FALSE);
        }
        else
        {
        	props.firePropertyChange(LayoutData.P_SOURCE_EXIST, null,Boolean.TRUE);
        }
    }

    @Override
    public void doPreDelete(LayoutData contents)
    {
    	List<AbstractConnectionModel> sources = new ArrayList<AbstractConnectionModel>(sourceConnections);
    	List<AbstractConnectionModel> targets = new ArrayList<AbstractConnectionModel>(targetConnections);		
		for (Iterator<AbstractConnectionModel> iter = sources.iterator(); iter.hasNext();) {
			AbstractConnectionModel model = iter.next();
			DeleteConnectionCommand delComm = new DeleteConnectionCommand();
			delComm.setConnectionModel(model);
			delComm.setContentsModel(contents);
			delComm.execute();
		}

		for (Iterator<AbstractConnectionModel> iter = targets.iterator(); iter.hasNext();) {
			AbstractConnectionModel model = iter.next();
			DeleteConnectionCommand delComm = new DeleteConnectionCommand();
			delComm.setConnectionModel(model);
			delComm.setContentsModel(contents);
			delComm.execute();
		}
    }
    
    public void doPostAdd()
    {
        LayoutData parent = (LayoutData) getParent();
        List<AbstractModel> children = new ArrayList<AbstractModel>(parent.getPartChildren());
        Iterator<AbstractModel> iter = children.iterator();
        while (iter.hasNext())
        {
            AbstractModel model = iter.next();
            if (model instanceof MultiTransitionModel)
            {
                ((MultiTransitionModel) model).doPostAdd();
            }
        }
    }

    @Override
	public boolean isPreview()
    {
        return ((LayoutData) getParent()).isPreview();
    }

    public void addSourceConnection(AbstractConnectionModel connx)
    {
        sourceConnections.add(connx);
        props.firePropertyChange(P_SOURCE_CONNECTION, null, null);
    }

    public void addTargetConnection(AbstractConnectionModel connx)
    {
        targetConnections.add(connx);
        props.firePropertyChange(P_TARGET_CONNECTION, null, null);
    }

    public Color getColor()
    {
        return colorRegistry.get(P_COLOR);
    }

    public Color getPreviewColor()
    {
        return colorRegistry.get(P_BLEND_COLOR);
    }

    public Rectangle getConstraint()
    {
        return constraint;
    }

    /**
     * @return Returns the id.
     */
    public String getId()
    {
        return id;
    }

    public String getLabel()
    {
        return label;
    }

    public List<AbstractConnectionModel> getConnectionsToFrom(StateNodeModel n) {
		List<AbstractConnectionModel> both = new ArrayList<AbstractConnectionModel>(32);
		both.addAll(getConnectionsTo(n));
		both.addAll(getConnectionsFrom(n));
		return both;
	}

	public List<AbstractConnectionModel> getConnectionsTo(StateNodeModel target)
    {
		List<AbstractConnectionModel> answ
	  	  = new ArrayList<AbstractConnectionModel>();
	  	
	  	for( AbstractConnectionModel t : targetConnections )
	  	{
	  		if(t.getTarget().equals(target)) answ.add(t);
	  	}
	  	
	  	return answ;
    }
	
	public List<AbstractConnectionModel> getConnectionsFrom(StateNodeModel source)
	{
		List<AbstractConnectionModel> answ
	  	  = new ArrayList<AbstractConnectionModel>();
	  	
	  	for( AbstractConnectionModel t : source.sourceConnections )
	  	{
	  		if(t.getTarget().equals(this)) answ.add(t);
	  	}
	  	
	  	return answ;
	}
    
    public List<AbstractConnectionModel> getModelSourceConnections()
    {
        return sourceConnections;
    }

    public List<AbstractConnectionModel> getModelTargetConnections()
    {
        return targetConnections;
    }

    @Override
	public IPropertyDescriptor[] getPropertyDescriptors()
    {
    	PropertyDescriptor idProp = new PropertyDescriptor(P_ID, "ID");
        PropertyDescriptor classProp = new PropertyDescriptor(P_CLASS,
                "Class");
        PropertyDescriptor intransProp = new PropertyDescriptor(
                P_INTRANS, "In");
        PropertyDescriptor outtransProp = new PropertyDescriptor(
                P_OUTTRANS, "Out");

        idProp.setCategory(PROPERTIES_MODEL_CATEGORY);
        classProp.setCategory(PROPERTIES_MODEL_CATEGORY);
        intransProp.setCategory(PROPERTIES_MODEL_CATEGORY);
        outtransProp.setCategory(PROPERTIES_MODEL_CATEGORY);
        
        PropertyDescriptor labelProp = new TextPropertyDescriptor(P_LABEL, "Label");
        PropertyDescriptor colorProp = new ColorPropertyDescriptor(P_COLOR, "Color");
        PropertyDescriptor shapeProp = new ComboBoxPropertyDescriptor(P_SHAPE, "Shape", shapeValues);
        
        labelProp.setCategory(PROPERTIES_APPEARANCE_CATEGORY);
        colorProp.setCategory(PROPERTIES_APPEARANCE_CATEGORY);
        shapeProp.setCategory(PROPERTIES_APPEARANCE_CATEGORY);
        
        IPropertyDescriptor[] descriptors = new IPropertyDescriptor[] { idProp,
                classProp, intransProp, outtransProp,
                labelProp, colorProp, shapeProp};
        
        return descriptors;
    }

    @Override
	public Object getPropertyValue(Object id)
    {
        if (id.equals(P_ID)) {
			return getId();
		} else if (id.equals(P_CLASS)) {
			if (getSource() != null)
				return getSource().getClassName();
		} else if (id.equals(P_LABEL)) {
			return getLabel();
		} else if (id.equals(P_COLOR)) {
			return getRGB();
		} else if (id.equals(P_SHAPE)) {
			return Integer.valueOf(shapeIndex);
		} else if (id.equals(P_OUTTRANS)) {
			String s = "";
			boolean first = true;
			for( AbstractConnectionModel out : sourceConnections ) {
				s += (first ? "" : "\n") + out.partialToString(this);
				first = false;
			}
			return s;
		} else if (id.equals(P_INTRANS)) {
			String s = "";
			boolean first = true;
			for( AbstractConnectionModel out : targetConnections ) {
				s += (first ? "" : "\n") + out.partialToString(this);
				first = false;
			}
			return s;
		}
		return null;
    }

    public RGB getRGB()
    {
        return colorRegistry.getRGB(P_COLOR);
    }

    /**
     * @return Returns the shape.
     */
    public String getShape()
    {
        return shape;
    }

    /**
     * @return Returns the source.
     */
    public SourceObjectModel getSource()
    {
        return source;
    }

    @Override
	public Content getXML()
    {
        Element content = new Element(IDTag.XML_state_tag);
        content.setAttribute(IDTag.XML_state_id, getId());
        //content.setAttribute(IDTag.XML_state_class,getSource().getClassName());
        content.setAttribute(IDTag.XML_state_label, getLabel());
        content.setAttribute(IDTag.XML_state_color, IDTag.RGB2Tag(getRGB()));
        Rectangle rect = getConstraint();
        content.setAttribute(IDTag.XML_state_top, "" + rect.y);
        content.setAttribute(IDTag.XML_state_left, "" + rect.x);
        content.setAttribute(IDTag.XML_state_width, "" + rect.width);
        content.setAttribute(IDTag.XML_state_height, "" + rect.height);
        content.setAttribute(IDTag.XML_state_shape, getShape());

        return content;
    }

    @Override
	public boolean isPropertySet(Object id)
    {
        if (id.equals(P_LABEL) || id.equals(P_COLOR) || id.equals(P_SHAPE))
                return true;
        return false;
    }

    public void removeSourceConnection(Object connx)
    {
        sourceConnections.remove(connx);
        props.firePropertyChange(P_SOURCE_CONNECTION, null, null);
    }

    public void removeTargetConnection(Object connx)
    {
        targetConnections.remove(connx);
        props.firePropertyChange(P_TARGET_CONNECTION, null, null);
    }

    private int getBlendColor(int oldValue)
    {
        int newCol = (int) Math.round(oldValue * 0.2) + 200;
        return (newCol > 255) ? 255 : newCol;
    }

    public void setColor(RGB color)
    {
        colorRegistry.put(P_COLOR, color);
        colorRegistry.put(P_BLEND_COLOR, new RGB(getBlendColor(color.red),
                getBlendColor(color.green), getBlendColor(color.blue)));
        props.firePropertyChange(P_COLOR, null, getColor());
    }

    public void setConstraint(Rectangle rect)
    {
        constraint = rect;
        props.firePropertyChange(P_CONSTRAINT, null, constraint);
    }

    public void setId(int id)
    {
    	stateID = id;
        setId("state" + Integer.toString(id));
    }
    
    public void setId(String text)
    {
        this.id = text;
    }

    public void setLabel(String text)
    {
        this.label = text;
        props.firePropertyChange(P_LABEL, null, text);
    }

    @Override
	public void setPropertyValue(Object id, Object value)
    {
        if (id.equals(P_ID))
        {
            setId((String) value);
        }
        else if (id.equals(P_CLASS))
        {
            //setClassName((String)value);
        }
        else if (id.equals(P_LABEL))
        {
            setLabel((String) value);
        }
        else if (id.equals(P_COLOR))
        {
            setColor((RGB) value);
        }
        else if (id.equals(P_SHAPE))
        {
            setShapeIndex(((Integer) value).intValue());
        }
    }

    /**
     * @param shape
     *            The shape to set.
     */
    public void setShape(String shape)
    {
        // verify shape style
        if (!shape.equals(shapeValues[shapeIndex]))
        {
            for (int i = 0; i < shapeValues.length; i++)
                if (shapeValues[i].equals(shape))
                {
                    shapeIndex = i;
                    break;
                }
        }
        if (!shape.equals(shapeValues[shapeIndex])) return;

        this.shape = shape;
        props.firePropertyChange(P_SHAPE, null, shape);
    }

    public void setShapeIndex(int index)
    {
        shapeIndex = index;
        setShape(shapeValues[index]);
    }

    /**
     * @param source
     *            The source to set.
     */
    public void setSource(SourceObjectModel source)
    {
        this.source = source;
    }
    
    public void setParent(LayoutData parent)
    {
    	this.parent = parent;
    }
    
    @Override
	public String toString()
    {
        return this.getId();
    }

    @Override
	public boolean equals(Object o)
    {
    	if( this == o ) return true;
    	if( !(o instanceof StateNodeModel) ) return false;
    	StateNodeModel snm = (StateNodeModel)o;
    	
    	String sid = snm.getId();
    	String mid = this.getId();
    	
    	if( sid == null || mid == null ) return false;
    	
    	return sid.equals(mid);
    }
    
    @Override
    public int hashCode()
    {
    	String id = getId();
    	
    	return ( id == null ? 0 : id.hashCode() );
    }
}